﻿using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.pictureBox.Paint += pictureBox_Paint;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 픽처 박스 페인트시 처리하기 - pictureBox_Paint(sender, e)

        /// <summary>
        /// 픽처 박스 페인트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pictureBox_Paint(object sender, PaintEventArgs e)
        {
          //DrawLEDLetter(e.Graphics);
          //DrawColorfulLEDText(e.Graphics);
          //DrawUnknownLEDLetter(e.Graphics);
            DrawLEDText(e.Graphics);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region LED 문자 그리기 - DrawLEDLetter(graphics)

        /// <summary>
        /// LED 문자 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        private void DrawLEDLetter(Graphics graphics)
        {
            graphics.Clear(this.pictureBox.BackColor);

            graphics.SmoothingMode = SmoothingMode.AntiAlias;

            const float margin       = 10;
            const float cellWidth    = 100;
            const float cellHeight   = 160;
            const float ledThickness = 15;
            const float gap          = 3;

            LEDText letter = new LEDText(cellWidth, cellHeight, ledThickness, gap);

            PointF position = new PointF(margin, margin);

            PointF[] pointArray00 = letter.GetLED00PointArray(position);
            PointF[] pointArray13 = letter.GetLED13PointArray(position);

            graphics.DrawPolygon(Pens.Red, pointArray00);

            graphics.DrawPolygon(Pens.Red, pointArray13);

            PointF[] pointArray01 = letter.GetLED01PointArray(position);
            PointF[] pointArray02 = letter.GetLED02PointArray(position);
            PointF[] pointArray04 = letter.GetLED04PointArray(position);
            PointF[] pointArray03 = letter.GetLED03PointArray(position);
            PointF[] pointArray05 = letter.GetLED05PointArray(position);
            PointF[] pointArray06 = letter.GetLED06PointArray(position);
            PointF[] pointArray07 = letter.GetLED07PointArray(position);
            PointF[] pointArray08 = letter.GetLED08PointArray(position);
            PointF[] pointArray09 = letter.GetLED09PointArray(position);
            PointF[] pointArray10 = letter.GetLED10PointArray(position);
            PointF[] pointArray11 = letter.GetLED11PointArray(position);
            PointF[] pointArray12 = letter.GetLED12PointArray(position);

            graphics.DrawPolygon(Pens.Red, pointArray01);
            graphics.DrawPolygon(Pens.Red, pointArray02);
            graphics.DrawPolygon(Pens.Red, pointArray03);
            graphics.DrawPolygon(Pens.Red, pointArray04);
            graphics.DrawPolygon(Pens.Red, pointArray05);
            graphics.DrawPolygon(Pens.Red, pointArray06);
            graphics.DrawPolygon(Pens.Red, pointArray07);
            graphics.DrawPolygon(Pens.Red, pointArray08);
            graphics.DrawPolygon(Pens.Red, pointArray09);
            graphics.DrawPolygon(Pens.Red, pointArray10);
            graphics.DrawPolygon(Pens.Red, pointArray11);
            graphics.DrawPolygon(Pens.Red, pointArray12);
        }

        #endregion
        #region 색상 LED 텍스트 그리기 - DrawColorfulLEDText(graphics)

        /// <summary>
        /// 색상 LED 텍스트 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        private void DrawColorfulLEDText(Graphics graphics)
        {
            graphics.Clear(Color.Black);

            graphics.SmoothingMode = SmoothingMode.AntiAlias;

            const float margin       = 10;
            const float cellWidth    = 200;
            const float cellHeight   = 320;
            const float ledThickness = 28;
            const float gap          = 5f;

            LEDText ledText = new LEDText(cellWidth, cellHeight, ledThickness, gap);

            Brush backgroundBrush1 = Brushes.Black;
            Brush usedBrush1       = Brushes.Yellow;
            Pen   usedPen1         = new Pen(Color.DarkRed, 3);
            Brush unusedBrush1     = new SolidBrush(Color.FromArgb(0, 40, 0));
            Pen   unusedPen1       = usedPen1;
            PointF position1       = new PointF(margin, margin);

            ledText.DrawText
            (
                graphics,
                backgroundBrush1,
                usedBrush1,
                usedPen1,
                unusedBrush1,
                unusedPen1,
                position1,
                1.2f,
                "3"
            );

            Brush  backgroundBrush2 = Brushes.Black;
            Brush  usedBrush2       = new HatchBrush(HatchStyle.HorizontalBrick, Color.Lime, Color.Green);
            Pen    usedPen2         = new Pen(Color.LightGreen, 3);
            Brush  unusedBrush2     = new SolidBrush(Color.FromArgb(0, 40, 0));
            Pen    unusedPen2       = Pens.Transparent;
            PointF position2        = new PointF(margin + ledText.CellWidth * 1.2f, margin);

            ledText.DrawText
            (
                graphics,
                backgroundBrush2,
                usedBrush2,
                usedPen2,
                unusedBrush2,
                unusedPen2,
                position2,
                1.2f,
                "5"
            );

            Size = new Size(505, 410);
        }

        #endregion
        #region 알 수 없는 문자 그리기 - DrawUnknownLEDLetter(graphics)

        /// <summary>
        /// 알 수 없는 문자 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        private void DrawUnknownLEDLetter(Graphics graphics)
        {
            graphics.Clear(Color.Black);

            graphics.SmoothingMode = SmoothingMode.AntiAlias;

            const float margin       = 10;
            const float cellWidth    = 200;
            const float cellHeight   = 320;
            const float ledThickness = 28;
            const float gap          = 1.5f;

            LEDText ledText = new LEDText(cellWidth, cellHeight, ledThickness, gap);

            Brush  backgroundBrush = Brushes.Black;
            Brush  usedBrush       = Brushes.Lime;
            Pen    usedPen         = Pens.Transparent;
            Brush  unusedBrush     = new SolidBrush(Color.FromArgb(0, 40, 0));
            Pen    unusedPen       = Pens.Transparent;
            PointF position        = new PointF(margin, margin);

            ledText.DrawText
            (
                graphics,
                backgroundBrush,
                usedBrush,
                usedPen,
                unusedBrush,
                unusedPen,
                position,
                1.2f,
                "!"
            );
        }

        #endregion
        #region LED 텍스트 그리기 - DrawLEDText(graphics)

        /// <summary>
        /// LED 텍스트 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        private void DrawLEDText(Graphics graphics)
        {
            graphics.Clear(Color.Black);

            graphics.SmoothingMode = SmoothingMode.AntiAlias;

            const float margin       = 10;
            const float cellWidth    = 50;
            const float cellHeight   = 80;
            const float ledThickness = 7;
            const float gap          = 1.5f;

            LEDText ledText = new LEDText(cellWidth, cellHeight, ledThickness, gap);

            Brush backgroundBrush = Brushes.Black;
            Brush usedBrush       = Brushes.Lime;
            Pen   usedPen         = Pens.Transparent;
            Brush unusedBrush     = new SolidBrush(Color.FromArgb(0, 40, 0));
            Pen   unusedPen       = Pens.Transparent;

            PointF position = new PointF(margin, margin);

            ledText.DrawText
            (
                graphics,
                backgroundBrush,
                usedBrush,
                usedPen,
                unusedBrush,
                unusedPen,
                position,
                1.2f,
                "ABCDEFGHI"
            );

            position.Y += ledText.CellHeight * 1.2f;

            ledText.DrawText
            (
                graphics,
                backgroundBrush,
                usedBrush,
                usedPen,
                unusedBrush,
                unusedPen,
                position,
                1.2f,
                "JKLMNOPQR"
            );

            position.Y += ledText.CellHeight * 1.2f;

            ledText.DrawText
            (
                graphics,
                backgroundBrush,
                usedBrush,
                usedPen,
                unusedBrush,
                unusedPen,
                position,
                1.2f,
                "STUVWXYZ0"
            );

            position.Y += ledText.CellHeight * 1.2f;

            ledText.DrawText
            (
                graphics,
                backgroundBrush,
                usedBrush,
                usedPen,
                unusedBrush,
                unusedPen,
                position,
                1.2f,
                "123456789"
            );
        }

        #endregion
    }
}